@@ -50,8 +50,14 @@
  * The above pipeline will re-encode an mp3 file into FLAC format and store
  * it on a remote host using the Samba protocol.
  * </para>
+ * <para>
+ * Applications can connect to the allow-overwrite signal to receive a callback when an
+ * existing file will be overwritten. The return value of the signal will determine if
+ * gnomevfssink will overwrite the resource or abort with an error.
+ * </para>
  * </refsect2>
  *
+ * Last reviewed on 2006-02-28 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -172,8 +178,9 @@ gst_gnome_vfs_sink_class_init (GstGnomeVFSSinkClass * klass)
   gobject_class->get_property = gst_gnome_vfs_sink_get_property;
   gobject_class->finalize = gst_gnome_vfs_sink_finalize;
 
-  gst_element_class_install_std_props (GST_ELEMENT_CLASS (klass),
-      "location", ARG_LOCATION, G_PARAM_READWRITE, NULL);
+  g_object_class_install_property (gobject_class, ARG_LOCATION,
+      g_param_spec_string ("location", "File Location",
+          "Location of the file to write", NULL, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_URI,
       g_param_spec_boxed ("uri", "GnomeVFSURI", "URI for GnomeVFS",
           GST_TYPE_GNOME_VFS_URI, G_PARAM_READWRITE));
@@ -182,6 +189,17 @@ gst_gnome_vfs_sink_class_init (GstGnomeVFSSinkClass * klass)
           "GnomeVFSHandle", "Handle for GnomeVFS",
           GST_TYPE_GNOME_VFS_HANDLE, G_PARAM_READWRITE));
 
+  /**
+   * GstGnomeVFSSink::allow-overwrite
+   * @sink: the object which received the signal
+   * @uri: the URI to be overwritten
+   *
+   * This signal is fired when gnomevfssink is about to overwrite an
+   * existing resource. The application can connect to this signal and ask
+   * the user if the resource may be overwritten. 
+   *
+   * Returns: A boolean indicating that the resource may be overwritten.
+   */
   gst_gnome_vfs_sink_signals[SIGNAL_ERASE_ASK] =
       g_signal_new ("allow-overwrite", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_CLEANUP, G_STRUCT_OFFSET (GstGnomeVFSSinkClass, erase_ask),
@@ -331,11 +349,13 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
   GnomeVFSResult result;
 
   if (sink->uri) {
-    /* open the file */
+    /* open the file, all permissions, umask will apply */
     result = gnome_vfs_create_uri (&(sink->handle), sink->uri,
         GNOME_VFS_OPEN_WRITE, TRUE,
-        GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE
-        | GNOME_VFS_PERM_GROUP_READ);
+        GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE |
+        GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_WRITE |
+        GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_WRITE);
+
     /* if the file existed and the property says to ask, then ask! */
     if (result == GNOME_VFS_ERROR_FILE_EXISTS) {
       gboolean erase_anyway = FALSE;
@@ -346,8 +366,9 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
       if (erase_anyway) {
         result = gnome_vfs_create_uri (&(sink->handle), sink->uri,
             GNOME_VFS_OPEN_WRITE, FALSE,
-            GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE
-            | GNOME_VFS_PERM_GROUP_READ);
+            GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE |
+            GNOME_VFS_PERM_GROUP_READ | GNOME_VFS_PERM_GROUP_WRITE |
+            GNOME_VFS_PERM_OTHER_READ | GNOME_VFS_PERM_OTHER_WRITE);
       }
     }
 
@@ -532,8 +553,11 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
     case GNOME_VFS_ERROR_NO_SPACE:{
       /* TODO: emit signal/send msg on out-of-diskspace and
        * handle this gracefully (see open bug) (tpm) */
+      GST_ELEMENT_ERROR (sink, RESOURCE, NO_SPACE_LEFT, (NULL),
+          ("bufsize=%u, written=%u", GST_BUFFER_SIZE (buf), (guint) written));
+      ret = GST_FLOW_ERROR;
+      break;
     }
-      /* fall-through */
     default:{
       gchar *filename = gnome_vfs_uri_to_string (sink->uri,
           GNOME_VFS_URI_HIDE_PASSWORD);
