@@ -57,8 +57,8 @@
 #include <gst/video/video.h>
 #include <time.h>
 
-static const GstElementDetails clock_overlay_details =
-GST_ELEMENT_DETAILS ("Clock overlay",
+static GstElementDetails clock_overlay_details =
+GST_ELEMENT_DETAILS ("Clock Overlay",
     "Filter/Editor/Video",
     "Overlays the current clock time on a video stream",
     "Tim-Philipp Müller <tim@centricular.net>");
@@ -76,26 +76,14 @@ GST_BOILERPLATE (GstClockOverlay, gst_clock_overlay, GstTextOverlay,
 static gchar *
 gst_clock_overlay_render_time (GstClockOverlay * overlay)
 {
-  struct tm *t;
+  struct tm t;
   time_t now;
 
-#ifdef HAVE_LOCALTIME_R
-  struct tm dummy;
-#endif
-
   now = time (NULL);
-
-#ifdef HAVE_LOCALTIME_R
-  t = localtime_r (&now, &dummy);
-#else
-  /* on win32 this apparently returns a per-thread struct which would be fine */
-  t = localtime (&now);
-#endif
-
-  if (t == NULL)
+  if (localtime_r (&now, &t) == NULL)
     return g_strdup ("--:--:--");
 
-  return g_strdup_printf ("%02u:%02u:%02u", t->tm_hour, t->tm_min, t->tm_sec);
+  return g_strdup_printf ("%02u:%02u:%02u", t.tm_hour, t.tm_min, t.tm_sec);
 }
 
 /* Called with lock held */
@@ -147,7 +135,7 @@ gst_clock_overlay_init (GstClockOverlay * overlay, GstClockOverlayClass * klass)
   pango_context_set_base_dir (context, PANGO_DIRECTION_LTR);
 
   font_description = pango_font_description_new ();
-  pango_font_description_set_family_static (font_description, "Monospace");
+  pango_font_description_set_family (font_description, g_strdup ("Monospace"));
   pango_font_description_set_style (font_description, PANGO_STYLE_NORMAL);
   pango_font_description_set_variant (font_description, PANGO_VARIANT_NORMAL);
   pango_font_description_set_weight (font_description, PANGO_WEIGHT_NORMAL);
