@@ -38,7 +38,7 @@
 /**
  * SECTION:gstcddabasesrc
  * @short_description: Base class for CD digital audio (CDDA) sources
- *
+ * 
  * <refsect2>
  * <para>
  * Provides a base class for CDDA sources, which handles things like seeking,
@@ -70,14 +70,6 @@
  * setting a new cdda://n+1 URI on playbin (as setting a new URI on playbin
  * involves closing and re-opening the CD device, which is much much slower).
  * </para>
- * <title>Tags and meta-information</title>
- * <para>
- * CDDA sources will automatically emit a number of tags, details about which
- * can be found in the libgsttag documentation. Those tags are:
- * #GST_TAG_CDDA_CDDB_DISCID, #GST_TAG_CDDA_CDDB_DISCID_FULL,
- * #GST_TAG_CDDA_MUSICBRAINZ_DISCID, #GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL,
- * among others.
- * </para>
  * </refsect2>
  */
 
@@ -86,7 +78,6 @@
 #endif
 
 #include <string.h>
-#include <stdlib.h>             /* for strtol */
 
 #include "gstcddabasesrc.h"
 #include "gst/gst-i18n-plugin.h"
@@ -164,7 +155,7 @@ static GType
 gst_cdda_base_src_mode_get_type (void)
 {
   static GType mode_type;       /* 0 */
-  static const GEnumValue modes[] = {
+  static GEnumValue modes[] = {
     {GST_CDDA_BASE_SRC_MODE_NORMAL, "Stream consists of a single track",
         "normal"},
     {GST_CDDA_BASE_SRC_MODE_CONTINUOUS, "Stream consists of the whole disc",
@@ -190,8 +181,23 @@ gst_cdda_base_src_base_init (gpointer g_class)
   track_format = gst_format_register ("track", "CD track");
   sector_format = gst_format_register ("sector", "CD sector");
 
-  /* register CDDA tags */
-  gst_tag_register_musicbrainz_tags ();
+  /* tags */
+  gst_tag_register (GST_TAG_CDDA_CDDB_DISCID, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "discid", "CDDB discid for metadata retrieval",
+      gst_tag_merge_use_first);
+
+  gst_tag_register (GST_TAG_CDDA_CDDB_DISCID_FULL, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "discid full",
+      "CDDB discid for metadata retrieval (full)", gst_tag_merge_use_first);
+
+  gst_tag_register (GST_TAG_CDDA_MUSICBRAINZ_DISCID, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "musicbrainz-discid",
+      "Musicbrainz discid for metadata retrieval", gst_tag_merge_use_first);
+
+  gst_tag_register (GST_TAG_CDDA_MUSICBRAINZ_DISCID_FULL, GST_TAG_FLAG_META,
+      G_TYPE_STRING, "musicbrainz-discid-full",
+      "Musicbrainz discid for metadata retrieval (full)",
+      gst_tag_merge_use_first);
 
 #if 0
   ///// FIXME: what type to use here? ///////
@@ -704,7 +710,7 @@ gst_cdda_base_src_do_seek (GstBaseSrc * basesrc, GstSegment * segment)
       seek_sector += src->tracks[0].start;
       break;
     default:
-      g_return_val_if_reached (FALSE);
+      g_assert_not_reached ();
   }
 
   src->cur_sector = (gint) seek_sector;
@@ -754,7 +760,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
         start_time = -1;
         break;
       default:
-        g_return_val_if_reached (FALSE);
+        g_assert_not_reached ();
     }
 
     switch (stop_type) {
@@ -779,7 +785,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
         stop_time = -1;
         break;
       default:
-        g_return_val_if_reached (FALSE);
+        g_assert_not_reached ();
     }
 
     GST_LOG_OBJECT (src, "seek segment %" GST_TIME_FORMAT "-%" GST_TIME_FORMAT,
@@ -810,7 +816,7 @@ gst_cdda_base_src_handle_track_seek (GstCddaBaseSrc * src, gdouble rate,
     return FALSE;
   }
 
-  GST_DEBUG_OBJECT (src, "seeking to track %" G_GINT64_FORMAT, start + 1);
+  GST_DEBUG_OBJECT (src, "seeking to track %d", start + 1);
 
   src->cur_sector = src->tracks[start].start;
   GST_DEBUG_OBJECT (src, "starting at sector %d", src->cur_sector);
@@ -884,7 +890,7 @@ gst_cdda_base_src_handle_event (GstBaseSrc * basesrc, GstEvent * event)
   return ret;
 }
 
-static GstURIType
+static guint
 gst_cdda_base_src_uri_get_type (void)
 {
   return GST_URI_SRC;
@@ -1025,10 +1031,7 @@ gst_cdda_base_src_add_track (GstCddaBaseSrc * src, GstCddaBaseSrcTrack * track)
   if (src->num_tracks > 0) {
     guint end_of_previous_track = src->tracks[src->num_tracks - 1].end;
 
-    if (track->start < end_of_previous_track) {
-      GST_WARNING ("track %2u overlaps with previous tracks", track->num);
-      return FALSE;
-    }
+    g_return_val_if_fail (track->start >= end_of_previous_track, FALSE);
   }
 
   GST_OBJECT_LOCK (src);
@@ -1484,7 +1487,6 @@ gst_cdda_base_src_stop (GstBaseSrc * basesrc)
   return TRUE;
 }
 
-
 static GstFlowReturn
 gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
 {
@@ -1493,10 +1495,8 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   GstBuffer *buf;
   GstFormat format;
   gboolean eos;
-
-  GstClockTime position = GST_CLOCK_TIME_NONE;
-  GstClockTime duration = GST_CLOCK_TIME_NONE;
-  gint64 qry_position;
+  gint64 position;
+  gint64 duration;
 
   g_assert (klass->read_sector != NULL);
 
@@ -1510,7 +1510,7 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
           src->cur_sector);
       break;
     default:
-      g_return_val_if_reached (GST_FLOW_ERROR);
+      g_assert_not_reached ();
   }
 
   if (eos) {
@@ -1550,23 +1550,13 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   }
 
   format = GST_FORMAT_TIME;
-  if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &qry_position)) {
-    gint64 next_ts = 0;
-
-    position = (GstClockTime) qry_position;
-
-    ++src->cur_sector;
-    if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &next_ts)) {
-      duration = (GstClockTime) (next_ts - qry_position);
-    }
-    --src->cur_sector;
+  if (!gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &position)) {
+    position = GST_CLOCK_TIME_NONE;
   }
 
-  /* fallback duration: 4 bytes per sample, 44100 samples per second */
-  if (duration == GST_CLOCK_TIME_NONE) {
-    duration = gst_util_uint64_scale_int (GST_BUFFER_SIZE (buf) >> 2,
-        GST_SECOND, 44100);
-  }
+  /* 4 bytes per sample, 44100 samples per second */
+  duration = gst_util_uint64_scale_int (GST_BUFFER_SIZE (buf) >> 2, GST_SECOND,
+      44100);
 
   GST_BUFFER_TIMESTAMP (buf) = position;
   GST_BUFFER_DURATION (buf) = duration;
