@@ -26,6 +26,13 @@
 #include "riff-ids.h"
 #include "riff-media.h"
 
+#include <gst/audio/multichannel.h>
+
+#include <string.h>
+
+GST_DEBUG_CATEGORY_EXTERN (riff_debug);
+#define GST_CAT_DEFAULT riff_debug
+
 /**
  * gst_riff_create_video_caps_with_data:
  * @codec_fcc: fourCC codec for this codec.
@@ -285,6 +292,12 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("FFmpeg MPEG-4");
       break;
 
+    case GST_MAKE_FOURCC ('3', 'i', 'v', 'd'):
+    case GST_MAKE_FOURCC ('3', 'I', 'V', 'D'):
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft MPEG-4 4.3");        /* FIXME? */
+      return gst_caps_from_string ("video/x-msmpeg, msmpegversion = (int) 43");
+
     case GST_MAKE_FOURCC ('3', 'I', 'V', '1'):
     case GST_MAKE_FOURCC ('3', 'I', 'V', '2'):
       caps = gst_caps_new_simple ("video/x-3ivx", NULL);
@@ -338,6 +351,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
           "msvideoversion", G_TYPE_INT, 1, NULL);
       if (codec_name)
         *codec_name = g_strdup ("MS video v1");
+      palette = strf_data;
+      strf_data = NULL;
       break;
 
     case GST_MAKE_FOURCC ('R', 'L', 'E', ' '):
@@ -494,6 +509,13 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         *codec_name = g_strdup ("PNG image");
       break;
 
+    case GST_MAKE_FOURCC ('F', 'L', 'V', '1'):
+      caps = gst_caps_new_simple ("video/x-flash-video",
+          "flvversion", G_TYPE_INT, 1, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Flash Video 1");
+      break;
+
     default:
       GST_WARNING ("Unknown video fourcc %" GST_FOURCC_FORMAT,
           GST_FOURCC_ARGS (codec_fcc));
@@ -525,24 +547,118 @@ gst_riff_create_video_caps (guint32 codec_fcc,
   }
 
   /* palette */
-  if (palette && GST_BUFFER_SIZE (palette) >= 256 * 4) {
-    GstBuffer *copy = gst_buffer_copy (palette);
+  if (palette) {
+    GstBuffer *copy;
+    guint num_colors;
+
+    if (strf != NULL)
+      num_colors = strf->num_colors;
+    else
+      num_colors = 256;
+
+    if (GST_BUFFER_SIZE (palette) >= (num_colors * 4)) {
+      /* palette is always at least 256*4 bytes */
+      copy = gst_buffer_new_and_alloc (MAX (num_colors * 4, 256 * 4));
+      memcpy (GST_BUFFER_DATA (copy), GST_BUFFER_DATA (palette),
+          GST_BUFFER_SIZE (palette));
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-    gint n;
-    guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
+      gint n;
+      guint32 *data = (guint32 *) GST_BUFFER_DATA (copy);
 
-    /* own endianness */
-    for (n = 0; n < 256; n++)
-      data[n] = GUINT32_FROM_LE (data[n]);
+      /* own endianness */
+      for (n = 0; n < num_colors; n++)
+        data[n] = GUINT32_FROM_LE (data[n]);
 #endif
-    gst_caps_set_simple (caps, "palette_data", GST_TYPE_BUFFER, copy, NULL);
-    gst_buffer_unref (copy);
+      gst_caps_set_simple (caps, "palette_data", GST_TYPE_BUFFER, copy, NULL);
+      gst_buffer_unref (copy);
+    } else {
+      GST_WARNING ("Palette smaller than expected: broken file");
+    }
   }
 
   return caps;
 }
 
+static const struct
+{
+  const guint32 ms_mask;
+  const GstAudioChannelPosition gst_pos;
+} layout_mapping[] = {
+  {
+  0x00001, GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT}, {
+  0x00002, GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT}, {
+  0x00004, GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER}, {
+  0x00008, GST_AUDIO_CHANNEL_POSITION_LFE}, {
+  0x00010, GST_AUDIO_CHANNEL_POSITION_REAR_LEFT}, {
+  0x00020, GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT}, {
+  0x00040, GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER}, {
+  0x00080, GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER}, {
+  0x00100, GST_AUDIO_CHANNEL_POSITION_REAR_CENTER}, {
+  0x00200, GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT}, {
+  0x00400, GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT}, {
+  0x00800, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_CENTER       */
+  {
+  0x01000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_FRONT_LEFT   */
+  {
+  0x02000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_FRONT_CENTER */
+  {
+  0x04000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_FRONT_RIGHT  */
+  {
+  0x08000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_BACK_LEFT    */
+  {
+  0x10000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_BACK_CENTER  */
+  {
+  0x20000, GST_AUDIO_CHANNEL_POSITION_INVALID}  /* TOP_BACK_RIGHT   */
+};
+
+#define MAX_CHANNEL_POSITIONS G_N_ELEMENTS (layout_mapping)
+
+static gboolean
+gst_riff_wavext_add_channel_layout (GstCaps * caps, guint32 layout)
+{
+  GstAudioChannelPosition pos[MAX_CHANNEL_POSITIONS];
+  GstStructure *s;
+  gint num_channels, i, p;
+
+  s = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_get_int (s, "channels", &num_channels))
+    g_return_val_if_reached (FALSE);
+
+  if (num_channels < 2 || num_channels > MAX_CHANNEL_POSITIONS) {
+    GST_DEBUG ("invalid number of channels: %d", num_channels);
+    return FALSE;
+  }
+
+  p = 0;
+  for (i = 0; i < MAX_CHANNEL_POSITIONS; ++i) {
+    if ((layout & layout_mapping[i].ms_mask) != 0) {
+      if (p >= num_channels) {
+        GST_WARNING ("More bits set in the channel layout map than there "
+            "are channels! Broken file");
+        return FALSE;
+      }
+      if (layout_mapping[i].gst_pos == GST_AUDIO_CHANNEL_POSITION_INVALID) {
+        GST_WARNING ("Unsupported channel position (mask 0x%08x) in channel "
+            "layout map - ignoring those channels", layout_mapping[i].ms_mask);
+        /* what to do? just ignore it and let downstream deal with a channel
+         * layout that has INVALID positions in it for now ... */
+      }
+      pos[p] = layout_mapping[i].gst_pos;
+      ++p;
+    }
+  }
+
+  if (p != num_channels) {
+    GST_WARNING ("Only %d bits set in the channel layout map, but there are "
+        "supposed to be %d channels! Broken file", p, num_channels);
+    return FALSE;
+  }
+
+  gst_audio_set_channel_positions (s, pos);
+  return TRUE;
+}
+
 GstCaps *
 gst_riff_create_audio_caps (guint16 codec_id,
     gst_riff_strh * strh, gst_riff_strf_auds * strf,
@@ -574,11 +690,40 @@ gst_riff_create_audio_caps (guint16 codec_id,
         gint ch = strf->channels;
         gint ws = strf->size;
 
-        caps = gst_caps_new_simple ("audio/x-raw-int",
-            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+        caps = gst_caps_new_simple ("audio/x-raw-int", "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, "channels", G_TYPE_INT, ch,   /* needed for _add_layout() */
             "width", G_TYPE_INT, (int) (ba * 8 / ch),
             "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, ws != 8, NULL);
+
+        /* Add default MS channel layout if we have more than 2 channels,
+         * but the layout isn't specified like with WAVEEXT below. Not sure
+         * if this is right, but at least it makes sound output work at all
+         * in those cases. Somebody with a a 5.1 setup should double-check
+         * with chan-id.wav */
+        if (ch > 2) {
+          guint32 channel_mask;
+
+          switch (ch) {
+            case 4:
+              channel_mask = 0x33;
+              break;
+            case 6:
+              channel_mask = 0x3f;
+              break;
+            default:
+              GST_WARNING ("don't know default layout for %d channels", ch);
+              channel_mask = 0;
+              break;
+          }
+
+          if (channel_mask) {
+            GST_DEBUG ("using default channel layout for %d channels", ch);
+            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+            }
+          }
+        }
       } else {
+        /* FIXME: this is pretty useless - we need fixed caps */
         caps = gst_caps_from_string ("audio/x-raw-int, "
             "endianness = (int) LITTLE_ENDIAN, "
             "signed = (boolean) { true, false }, "
@@ -679,7 +824,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
 
       if (strf != NULL) {
         gst_caps_set_simple (caps,
-            "bitrate", G_TYPE_INT, strf->av_bps * 8, NULL);
+            "bitrate", G_TYPE_INT, strf->av_bps * 8,
+            "depth", G_TYPE_INT, strf->size, NULL);
       } else {
         gst_caps_set_simple (caps,
             "bitrate", GST_TYPE_INT_RANGE, 0, G_MAXINT, NULL);
@@ -692,6 +838,74 @@ gst_riff_create_audio_caps (guint16 codec_id,
         *codec_name = g_strdup ("Sony ATRAC3");
       break;
 
+    case GST_RIFF_WAVE_FORMAT_EXTENSIBLE:{
+      guint16 valid_bits_per_sample;
+      guint32 channel_mask;
+      guint32 subformat_guid[4];
+      const guint8 *data;
+
+      if (GST_BUFFER_SIZE (strf_data) != 22) {
+        GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %d (expected: 22)",
+            GST_BUFFER_SIZE (strf_data));
+        return NULL;
+      }
+
+      data = GST_BUFFER_DATA (strf_data);
+      valid_bits_per_sample = GST_READ_UINT16_LE (data);
+      channel_mask = GST_READ_UINT32_LE (data + 2);
+      subformat_guid[0] = GST_READ_UINT32_LE (data + 6);
+      subformat_guid[1] = GST_READ_UINT32_LE (data + 10);
+      subformat_guid[2] = GST_READ_UINT32_LE (data + 14);
+      subformat_guid[3] = GST_READ_UINT32_LE (data + 18);
+
+      GST_DEBUG ("valid bps    = %u", valid_bits_per_sample);
+      GST_DEBUG ("channel mask = 0x%08x", channel_mask);
+      GST_DEBUG ("GUID         = %08x-%08x-%08x-%08x", subformat_guid[0],
+          subformat_guid[1], subformat_guid[2], subformat_guid[3]);
+
+      if (subformat_guid[1] == 0x00100000 &&
+          subformat_guid[2] == 0xaa000080 && subformat_guid[3] == 0x719b3800) {
+        if (subformat_guid[0] == 0x00000001) {
+          GST_DEBUG ("PCM");
+          if (strf != NULL) {
+            gint ba = strf->blockalign;
+            gint ws = strf->size;
+            gint depth = ws;
+
+            if (valid_bits_per_sample != 0)
+              depth = valid_bits_per_sample;
+
+            caps = gst_caps_new_simple ("audio/x-raw-int",
+                "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+                "channels", G_TYPE_INT, strf->channels,
+                "width", G_TYPE_INT, (int) (ba * 8 / strf->channels),
+                "depth", G_TYPE_INT, depth,
+                "rate", G_TYPE_INT, strf->rate,
+                "signed", G_TYPE_BOOLEAN, (depth > 8) ? TRUE : FALSE, NULL);
+
+            if (!gst_riff_wavext_add_channel_layout (caps, channel_mask)) {
+              GST_WARNING ("failed to add channel layout");
+              gst_caps_unref (caps);
+              caps = NULL;
+            }
+            rate_chan = FALSE;
+
+            if (codec_name) {
+              *codec_name = g_strdup_printf ("Uncompressed %d-bit PCM audio",
+                  strf->size);
+            }
+          }
+        } else if (subformat_guid[0] == 0x00000003) {
+          GST_DEBUG ("FIXME: handle IEEE float format");
+        }
+
+        if (caps == NULL) {
+          GST_WARNING ("Unknown WAVE_FORMAT_EXTENSIBLE audio format");
+          return NULL;
+        }
+      }
+      break;
+    }
     default:
       GST_WARNING ("Unknown audio tag 0x%04x", codec_id);
       return NULL;
@@ -807,6 +1021,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('S', 'P', '5', '3'),
     GST_MAKE_FOURCC ('P', 'N', 'G', ' '),
     GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'),
+    GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
     /* FILL ME */
     0
   };
