@@ -45,35 +45,77 @@ static gboolean xml_check_first_element (GstTypeFind * tf,
 static GstStaticCaps utf8_caps = GST_STATIC_CAPS ("text/plain");
 
 #define UTF8_CAPS gst_static_caps_get(&utf8_caps)
-static void
-utf8_type_find (GstTypeFind * tf, gpointer unused)
+
+static gboolean
+utf8_type_find_have_valid_utf8_at_offset (GstTypeFind * tf, guint64 offset,
+    GstTypeFindProbability * prob)
 {
   guint8 *data;
 
   /* randomly decided values */
-  guint size = 1024;            /* starting size */
+  guint min_size = 16;          /* minimum size  */
+  guint size = 32 * 1024;       /* starting size */
   guint probability = 95;       /* starting probability */
   guint step = 10;              /* how much we reduce probability in each
                                  * iteration */
 
-  /* leave xml to the xml typefinders */
-  if (xml_check_first_element (tf, "", 0))
-    return;
-
-  while (probability > step) {
-    data = gst_type_find_peek (tf, 0, size);
+  while (probability > step && size > min_size) {
+    data = gst_type_find_peek (tf, offset, size);
     if (data) {
       gchar *end;
       gchar *start = (gchar *) data;
 
       if (g_utf8_validate (start, size, (const gchar **) &end) || (end - start + 4 > size)) {   /* allow last char to be cut off */
-        gst_type_find_suggest (tf, probability, UTF8_CAPS);
+        *prob = probability;
+        return TRUE;
       }
-      return;
+      *prob = 0;
+      return FALSE;
     }
     size /= 2;
     probability -= step;
   }
+  *prob = 0;
+  return FALSE;
+}
+
+static void
+utf8_type_find (GstTypeFind * tf, gpointer unused)
+{
+  GstTypeFindProbability start_prob, mid_prob;
+  guint64 length;
+
+  /* leave xml to the xml typefinders */
+  if (xml_check_first_element (tf, "", 0))
+    return;
+
+  /* check beginning of stream */
+  if (!utf8_type_find_have_valid_utf8_at_offset (tf, 0, &start_prob))
+    return;
+
+  GST_LOG ("start is plain text with probability of %u", start_prob);
+
+  /* POSSIBLE is the highest probability we ever return if we can't
+   * probe into the middle of the file and don't know its length */
+
+  length = gst_type_find_get_length (tf);
+  if (length == 0 || length == (guint64) - 1) {
+    gst_type_find_suggest (tf, MIN (start_prob, GST_TYPE_FIND_POSSIBLE),
+        UTF8_CAPS);
+    return;
+  }
+
+  if (length < 64 * 1024) {
+    gst_type_find_suggest (tf, start_prob, UTF8_CAPS);
+    return;
+  }
+
+  /* check middle of stream */
+  if (!utf8_type_find_have_valid_utf8_at_offset (tf, length / 2, &mid_prob))
+    return;
+
+  GST_LOG ("middle is plain text with probability of %u", mid_prob);
+  gst_type_find_suggest (tf, (start_prob + mid_prob) / 2, UTF8_CAPS);
 }
 
 /*** text/uri-list ***/
@@ -272,7 +314,7 @@ id3_type_find (GstTypeFind * tf, gpointer unused)
   }
   data = gst_type_find_peek (tf, -128, 3);
   if (data && memcmp (data, "TAG", 3) == 0) {
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, ID3_CAPS);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 3, ID3_CAPS);
   }
 }
 
@@ -289,14 +331,14 @@ apetag_type_find (GstTypeFind * tf, gpointer unused)
   /* APEv1/2 at start of file */
   data = gst_type_find_peek (tf, 0, 8);
   if (data && !memcmp (data, "APETAGEX", 8)) {
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APETAG_CAPS);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, APETAG_CAPS);
     return;
   }
 
   /* APEv1/2 at end of file */
   data = gst_type_find_peek (tf, -32, 8);
   if (data && !memcmp (data, "APETAGEX", 8)) {
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APETAG_CAPS);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 2, APETAG_CAPS);
     return;
   }
 }
@@ -540,154 +582,202 @@ static GstStaticCaps mp3_caps = GST_STATIC_CAPS ("audio/mpeg, "
 #define GST_MP3_TYPEFIND_SYNC_SIZE (2048)
 
 static void
-mp3_type_find (GstTypeFind * tf, gpointer unused)
+mp3_type_find_at_offset (GstTypeFind * tf, guint64 start_off,
+    guint * found_layer, GstTypeFindProbability * found_prob)
 {
-  guint64 length = gst_type_find_get_length (tf);
-  gint try;
   guint8 *data = NULL;
+  guint8 *data_end = NULL;
   guint size;
   guint64 skipped;
+  gint last_free_offset = -1;
+  gint last_free_framelen = -1;
 
-  for (try = 0; try < 2; try++) {
-    gint last_free_offset = -1, last_free_framelen = -1;
-    guint64 start_off = (try == 0) ? 0 : length / 2;
+  *found_layer = 0;
+  *found_prob = 0;
 
-    if (try != 0 && start_off == 0)
-      break;
-
-    size = 0;
-    skipped = 0;
-    while (skipped < GST_MP3_TYPEFIND_TRY_SYNC) {
-      if (size <= 0) {
-        size = GST_MP3_TYPEFIND_SYNC_SIZE * 2;
-        do {
-          size /= 2;
-          data = gst_type_find_peek (tf, skipped + start_off, size);
-        } while (size > 10 && !data);
-        if (!data)
-          break;
-      }
-      if (*data == 0xFF) {
-        guint8 *head_data = NULL;
-        guint layer = 0, bitrate, samplerate, channels;
-        guint found = 0;        /* number of valid headers found */
-        guint64 offset = skipped;
-
-        while (found < GST_MP3_TYPEFIND_TRY_HEADERS) {
-          guint32 head;
-          guint length;
-          guint prev_layer = 0, prev_bitrate = 0,
-              prev_channels = 0, prev_samplerate = 0;
-          gboolean free = FALSE;
-
-          if (offset + 4 <= skipped + size) {
-            head_data = data + offset - skipped;
-          } else {
-            head_data = gst_type_find_peek (tf, offset + start_off, 4);
-          }
-          if (!head_data)
-            break;
-          head = GST_READ_UINT32_BE (head_data);
-          if (!(length = mp3_type_frame_length_from_header (head, &layer,
-                      &channels, &bitrate, &samplerate, &free,
-                      last_free_framelen))) {
-            if (free) {
-              if (last_free_offset == -1)
-                last_free_offset = offset;
-              else {
-                last_free_framelen = offset - last_free_offset;
-                offset = last_free_offset;
-                continue;
-              }
-            } else {
-              last_free_framelen = -1;
-            }
-
-            GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
-                " (0x%X) was not an mp3 header (possibly-free: %s)",
-                found + 1, offset, (guint) offset, free ? "yes" : "no");
-            break;
-          }
-          if ((prev_layer && prev_layer != layer) ||
-              /* (prev_bitrate && prev_bitrate != bitrate) || <-- VBR */
-              (prev_samplerate && prev_samplerate != samplerate) ||
-              (prev_channels && prev_channels != channels)) {
-            /* this means an invalid property, or a change, which might mean
-             * that this is not a mp3 but just a random bytestream. It could
-             * be a freaking funky encoded mp3 though. We'll just not count
-             * this header*/
-            prev_layer = layer;
-            prev_bitrate = bitrate;
-            prev_channels = channels;
-            prev_samplerate = samplerate;
-          } else {
-            found++;
-            GST_LOG ("found %d. header at offset %" G_GUINT64_FORMAT " (0x%X)",
-                found, offset, (guint) offset);
-          }
-          offset += length;
-        }
-        g_assert (found <= GST_MP3_TYPEFIND_TRY_HEADERS);
-        if (found == GST_MP3_TYPEFIND_TRY_HEADERS ||
-            (found >= GST_MP3_TYPEFIND_MIN_HEADERS && head_data == NULL)) {
-          /* we can make a valid guess */
-          guint probability = found * GST_TYPE_FIND_MAXIMUM *
-              (GST_MP3_TYPEFIND_TRY_SYNC - skipped) /
-              GST_MP3_TYPEFIND_TRY_HEADERS / GST_MP3_TYPEFIND_TRY_SYNC;
-
-          if (probability < GST_TYPE_FIND_MINIMUM)
-            probability = GST_TYPE_FIND_MINIMUM;
-          probability /= (try + 1);
-          GST_INFO
-              ("audio/mpeg calculated %u  =  %u  *  %u / %u  *  (%u - %u) / %u",
-              probability, GST_TYPE_FIND_MAXIMUM, found,
-              GST_MP3_TYPEFIND_TRY_HEADERS, GST_MP3_TYPEFIND_TRY_SYNC, skipped,
-              GST_MP3_TYPEFIND_TRY_SYNC);
-          /* make sure we're not id3 tagged */
-          head_data = gst_type_find_peek (tf, -128, 3);
-          if (!head_data) {
-            probability = probability * 4 / 5;
-          } else if (memcmp (head_data, "TAG", 3) == 0) {
-            probability = 0;
-          }
-          g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
-          if (probability > 0) {
-            GstCaps *caps;
-
-            g_assert (layer > 0);
-            caps = gst_caps_copy (MP3_CAPS);
-            gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
-                G_TYPE_INT, layer, NULL);
-            gst_type_find_suggest (tf, probability, caps);
-            gst_caps_unref (caps);
-            return;
-          }
-          goto no_luck;
-        }
-      }
-      data++;
-      skipped++;
-      size--;
+  size = 0;
+  skipped = 0;
+  while (skipped < GST_MP3_TYPEFIND_TRY_SYNC) {
+    if (size <= 0) {
+      size = GST_MP3_TYPEFIND_SYNC_SIZE * 2;
+      do {
+        size /= 2;
+        data = gst_type_find_peek (tf, skipped + start_off, size);
+      } while (size > 10 && !data);
+      if (!data)
+        break;
+      data_end = data + size;
     }
+    if (*data == 0xFF) {
+      guint8 *head_data = NULL;
+      guint layer = 0, bitrate, samplerate, channels;
+      guint found = 0;          /* number of valid headers found */
+      guint64 offset = skipped;
+
+      while (found < GST_MP3_TYPEFIND_TRY_HEADERS) {
+        guint32 head;
+        guint length;
+        guint prev_layer = 0, prev_bitrate = 0;
+        guint prev_channels = 0, prev_samplerate = 0;
+        gboolean free = FALSE;
+
+        if ((gint64) (offset - skipped + 4) >= 0 &&
+            data + offset - skipped + 4 < data_end) {
+          head_data = data + offset - skipped;
+        } else {
+          head_data = gst_type_find_peek (tf, offset + start_off, 4);
+        }
+        if (!head_data)
+          break;
+        head = GST_READ_UINT32_BE (head_data);
+        if (!(length = mp3_type_frame_length_from_header (head, &layer,
+                    &channels, &bitrate, &samplerate, &free,
+                    last_free_framelen))) {
+          if (free) {
+            if (last_free_offset == -1)
+              last_free_offset = offset;
+            else {
+              last_free_framelen = offset - last_free_offset;
+              offset = last_free_offset;
+              continue;
+            }
+          } else {
+            last_free_framelen = -1;
+          }
+
+          GST_LOG ("%d. header at offset %" G_GUINT64_FORMAT
+              " (0x%" G_GINT64_MODIFIER "x) was not an mp3 header "
+              "(possibly-free: %s)", found + 1, start_off + offset,
+              start_off + offset, free ? "yes" : "no");
+          break;
+        }
+        if ((prev_layer && prev_layer != layer) ||
+            /* (prev_bitrate && prev_bitrate != bitrate) || <-- VBR */
+            (prev_samplerate && prev_samplerate != samplerate) ||
+            (prev_channels && prev_channels != channels)) {
+          /* this means an invalid property, or a change, which might mean
+           * that this is not a mp3 but just a random bytestream. It could
+           * be a freaking funky encoded mp3 though. We'll just not count
+           * this header*/
+          prev_layer = layer;
+          prev_bitrate = bitrate;
+          prev_channels = channels;
+          prev_samplerate = samplerate;
+        } else {
+          found++;
+          GST_LOG ("found %d. header at offset %" G_GUINT64_FORMAT " (0x%"
+              G_GINT64_MODIFIER "X)", found, start_off + offset,
+              start_off + offset);
+        }
+        offset += length;
+      }
+      g_assert (found <= GST_MP3_TYPEFIND_TRY_HEADERS);
+      if (found == GST_MP3_TYPEFIND_TRY_HEADERS ||
+          (found >= GST_MP3_TYPEFIND_MIN_HEADERS && head_data == NULL)) {
+        /* we can make a valid guess */
+        guint probability = found * GST_TYPE_FIND_MAXIMUM *
+            (GST_MP3_TYPEFIND_TRY_SYNC - skipped) /
+            GST_MP3_TYPEFIND_TRY_HEADERS / GST_MP3_TYPEFIND_TRY_SYNC;
+
+        if (probability < GST_TYPE_FIND_MINIMUM)
+          probability = GST_TYPE_FIND_MINIMUM;
+        if (start_off > 0)
+          probability /= 2;
+
+        GST_INFO
+            ("audio/mpeg calculated %u  =  %u  *  %u / %u  *  (%u - %u) / %u",
+            probability, GST_TYPE_FIND_MAXIMUM, found,
+            GST_MP3_TYPEFIND_TRY_HEADERS, GST_MP3_TYPEFIND_TRY_SYNC, skipped,
+            GST_MP3_TYPEFIND_TRY_SYNC);
+        /* make sure we're not id3 tagged */
+        head_data = gst_type_find_peek (tf, -128, 3);
+        if (!head_data) {
+          probability = probability * 4 / 5;
+        } else if (memcmp (head_data, "TAG", 3) == 0) {
+          probability = 0;
+        }
+        g_assert (probability <= GST_TYPE_FIND_MAXIMUM);
+
+        *found_prob = probability;
+        if (probability > 0)
+          *found_layer = layer;
+        return;
+      }
+    }
+    data++;
+    skipped++;
+    size--;
+  }
+}
+
+static void
+mp3_type_find (GstTypeFind * tf, gpointer unused)
+{
+  GstTypeFindProbability prob, mid_prob;
+  guint8 *data;
+  guint layer, mid_layer;
+  guint64 length;
+
+  mp3_type_find_at_offset (tf, 0, &layer, &prob);
+  length = gst_type_find_get_length (tf);
+
+  if (length == 0 || length == (guint64) - 1) {
+    if (prob != 0)
+      goto suggest;
+    return;
   }
 
-no_luck:
+  /* if we're pretty certain already, skip the additional check */
+  if (prob >= GST_TYPE_FIND_LIKELY)
+    goto suggest;
 
-  /* no luck so far, let's see if there's a valid header right at the start */
-  data = gst_type_find_peek (tf, 0, 4); /* use min. frame size? */
-  if (data) {
-    GstCaps *caps;
-    guint layer;
+  mp3_type_find_at_offset (tf, length / 2, &mid_layer, &mid_prob);
 
-    if (mp3_type_frame_length_from_header (GST_READ_UINT32_BE (data),
-            &layer, NULL, NULL, NULL, NULL, 0) != 0) {
-      caps = gst_caps_copy (MP3_CAPS);
-      gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
-          G_TYPE_INT, layer, NULL);
-      GST_LOG ("possible mpeg audio layer %u frame at offset 0", layer);
-      gst_type_find_suggest (tf, GST_TYPE_FIND_POSSIBLE - 10, caps);
-      gst_caps_unref (caps);
+  if (mid_prob > 0) {
+    if (prob == 0) {
+      GST_LOG ("detected audio/mpeg only in the middle (p=%u)", mid_prob);
+      layer = mid_layer;
+      prob = mid_prob;
+      goto suggest;
     }
+
+    if (layer != mid_layer) {
+      GST_WARNING ("audio/mpeg layer discrepancy: %u vs. %u", layer, mid_layer);
+      return;                   /* FIXME: or should we just go with the one in the middle? */
+    }
+
+    /* detected mpeg audio both in middle of the file and at the start */
+    prob = (prob + mid_prob) / 2;
+    goto suggest;
+  }
+
+  /* let's see if there's a valid header right at the start */
+  data = gst_type_find_peek (tf, 0, 4); /* use min. frame size? */
+  if (data && mp3_type_frame_length_from_header (GST_READ_UINT32_BE (data),
+          &layer, NULL, NULL, NULL, NULL, 0) != 0) {
+    if (prob == 0)
+      prob = GST_TYPE_FIND_POSSIBLE - 10;
+    else
+      prob = MAX (GST_TYPE_FIND_POSSIBLE - 10, prob + 10);
+  }
+
+  if (prob > 0)
+    goto suggest;
+
+  return;
+
+suggest:
+  {
+    GstCaps *caps;
+
+    g_assert (layer > 0);
+
+    caps = gst_caps_make_writable (MP3_CAPS);
+    gst_structure_set (gst_caps_get_structure (caps, 0), "layer",
+        G_TYPE_INT, layer, NULL);
+    gst_type_find_suggest (tf, prob, caps);
+    gst_caps_unref (caps);
+    return;
   }
 }
 
@@ -982,6 +1072,30 @@ mpeg1_sys_type_find (GstTypeFind * tf, gpointer unused)
   }
 }
 
+/*** video/mpeg MPEG-4 elementary video stream ***/
+
+static GstStaticCaps mpeg4_video_caps = GST_STATIC_CAPS ("video/mpeg, "
+    "systemstream = (boolean) false, mpegversion = 4");
+#define MPEG4_VIDEO_CAPS gst_static_caps_get(&mpeg4_video_caps)
+static void
+mpeg4_video_type_find (GstTypeFind * tf, gpointer unused)
+{
+  /* Header is a video object start code followed by a video object layer
+   * start code. The last byte of this 8-byte header can be from 0x20 - 0x2F */
+  static const guint8 header[] = { 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x01 };
+  guint8 *data = NULL;
+
+  data = gst_type_find_peek (tf, 0, 8);
+
+  if (data && memcmp (data, header, 7) == 0 &&
+      data[7] >= 0x20 && data[7] <= 0x2F) {
+    GstCaps *caps = gst_caps_copy (MPEG4_VIDEO_CAPS);
+
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM - 1, caps);
+    gst_caps_unref (caps);
+  }
+}
+
 /*** video/mpeg video stream ***/
 
 static GstStaticCaps mpeg_video_caps = GST_STATIC_CAPS ("video/mpeg, "
@@ -1156,7 +1270,7 @@ ape_type_find (GstTypeFind * tf, gpointer unused)
   guint8 *data = gst_type_find_peek (tf, 0, 4);
 
   if (data && memcmp (data, "MAC ", 4) == 0) {
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, APE_CAPS);
+    gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY + 10, APE_CAPS);
   }
 }
 
@@ -1190,14 +1304,47 @@ static GstStaticCaps q3gp_caps = GST_STATIC_CAPS ("application/x-3gp");
 static void
 q3gp_type_find (GstTypeFind * tf, gpointer unused)
 {
-  guint8 *data = gst_type_find_peek (tf, 4, 8);
 
-  if (data &&
-      (memcmp (data, "ftyp3gp4", 8) == 0 ||
-          memcmp (data, "ftyp3gp5", 8) == 0 ||
-          memcmp (data, "ftyp3gp6", 8) == 0)) {
-    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, Q3GP_CAPS);
+  guint32 ftyp_size = 0;
+  gint offset = 0;
+  guint8 *data = NULL;
+
+  if ((data = gst_type_find_peek (tf, 0, 12)) == NULL) {
+    return;
   }
+
+  data += 4;
+  if (memcmp (data, "ftyp", 4) != 0) {
+    return;
+  }
+
+  /* check major brand */
+  data += 4;
+  if (memcmp (data, "3gp", 3) == 0 ||
+      memcmp (data, "3gr", 3) == 0 ||
+      memcmp (data, "3gs", 3) == 0 || memcmp (data, "3gg", 3) == 0) {
+    gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, Q3GP_CAPS);
+    return;
+  }
+
+  /* check compatible brands */
+  if ((data = gst_type_find_peek (tf, 0, 4)) != NULL) {
+    ftyp_size = GST_READ_UINT32_BE (data);
+  }
+  for (offset = 16; offset < ftyp_size; offset += 4) {
+    if ((data = gst_type_find_peek (tf, offset, 3)) == NULL) {
+      break;
+    }
+    if (memcmp (data, "3gp", 3) == 0 ||
+        memcmp (data, "3gr", 3) == 0 ||
+        memcmp (data, "3gs", 3) == 0 || memcmp (data, "3gg", 3) == 0) {
+      gst_type_find_suggest (tf, GST_TYPE_FIND_LIKELY, Q3GP_CAPS);
+      break;
+    }
+  }
+
+  return;
+
 }
 
 /*** video/quicktime ***/
@@ -1216,22 +1363,27 @@ qt_type_find (GstTypeFind * tf, gpointer unused)
   guint64 size;
 
   while ((data = gst_type_find_peek (tf, offset, 8)) != NULL) {
-    if (STRNCMP (&data[4], "wide", 4) != 0 &&
-        STRNCMP (&data[4], "moov", 4) != 0 &&
-        STRNCMP (&data[4], "mdat", 4) != 0 &&
-        STRNCMP (&data[4], "pnot", 4) != 0 &&
-        STRNCMP (&data[4], "PICT", 4) != 0 &&
-        STRNCMP (&data[4], "ftyp", 4) != 0 &&
-        STRNCMP (&data[4], "free", 4) != 0 &&
-        STRNCMP (&data[4], "skip", 4) != 0) {
-      tip = 0;
-      break;
+    /* box/atom types that are in common with ISO base media file format */
+    if (STRNCMP (&data[4], "moov", 4) == 0 ||
+        STRNCMP (&data[4], "mdat", 4) == 0 ||
+        STRNCMP (&data[4], "ftyp", 4) == 0 ||
+        STRNCMP (&data[4], "free", 4) == 0 ||
+        STRNCMP (&data[4], "skip", 4) == 0) {
+      if (tip == 0) {
+        tip = GST_TYPE_FIND_LIKELY;
+      } else {
+        tip = GST_TYPE_FIND_NEARLY_CERTAIN;
+      }
     }
-    if (tip == 0) {
-      tip = GST_TYPE_FIND_LIKELY;
-    } else {
+    /* other box/atom types, apparently quicktime specific */
+    else if (STRNCMP (&data[4], "pnot", 4) == 0 ||
+        STRNCMP (&data[4], "PICT", 4) == 0 ||
+        STRNCMP (&data[4], "wide", 4) == 0) {
       tip = GST_TYPE_FIND_MAXIMUM;
       break;
+    } else {
+      tip = 0;
+      break;
     }
     size = GST_READ_UINT32_BE (data);
     if (size == 1) {
@@ -1770,7 +1922,7 @@ speex_type_find (GstTypeFind * tf, gpointer private)
 
 /*** application/x-ogg-skeleton ***/
 static GstStaticCaps ogg_skeleton_caps =
-GST_STATIC_CAPS ("application/x-ogg-skeleton");
+GST_STATIC_CAPS ("application/x-ogg-skeleton, parsed=(boolean)FALSE");
 #define OGG_SKELETON_CAPS (gst_static_caps_get(&ogg_skeleton_caps))
 static void
 oggskel_type_find (GstTypeFind * tf, gpointer private)
@@ -1800,9 +1952,11 @@ static GstStaticCaps cmml_caps = GST_STATIC_CAPS ("text/x-cmml");
 static void
 cmml_type_find (GstTypeFind * tf, gpointer private)
 {
+  /* Header is 12 bytes minimum (though we don't check the minor version */
   guint8 *data = gst_type_find_peek (tf, 0, 12);
 
   if (data) {
+
     /* 8 byte string "CMML\0\0\0\0" for the magic number */
     if (memcmp (data, "CMML\0\0\0\0", 8) != 0)
       return;
@@ -1811,10 +1965,6 @@ cmml_type_find (GstTypeFind * tf, gpointer private)
     /* Require that the header contains at least version 2.0 */
     if (GST_READ_UINT16_LE (data) < 2)
       return;
-    data += 2;
-
-    if (GST_READ_UINT16_LE (data) != 0)
-      return;
 
     gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, CMML_CAPS);
   }
@@ -2094,6 +2244,8 @@ plugin_init (GstPlugin * plugin)
   static gchar *msdos_exts[] = { "dll", "exe", "ocx", "sys", "scr",
     "msstyles", "cpl", NULL
   };
+  static gchar *flv_exts[] = { "flv", NULL };
+  static gchar *m4v_exts[] = { "m4v", NULL };
 
   GST_DEBUG_CATEGORY_INIT (type_find_debug, "typefindfunctions",
       GST_DEBUG_FG_GREEN | GST_DEBUG_BG_RED, "generic type find functions");
@@ -2105,7 +2257,7 @@ plugin_init (GstPlugin * plugin)
       GST_TYPE_FIND_MAXIMUM);
   /* -1 so id3v1 or apev1/2 are detected with higher preference */
   TYPE_FIND_REGISTER_START_WITH (plugin, "audio/x-musepack", GST_RANK_PRIMARY,
-      musepack_exts, "MP+", 3, GST_TYPE_FIND_MAXIMUM - 1);
+      musepack_exts, "MP+", 3, GST_TYPE_FIND_LIKELY + 10);
   TYPE_FIND_REGISTER (plugin, "audio/x-au", GST_RANK_MARGINAL,
       au_type_find, au_exts, AU_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER_RIFF (plugin, "video/x-msvideo", GST_RANK_PRIMARY,
@@ -2119,9 +2271,9 @@ plugin_init (GstPlugin * plugin)
       flac_exts, "fLaC", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "video/x-fli", GST_RANK_MARGINAL, flx_type_find,
       flx_exts, FLX_CAPS, NULL, NULL);
-  TYPE_FIND_REGISTER (plugin, "application/x-id3", GST_RANK_PRIMARY,
+  TYPE_FIND_REGISTER (plugin, "application/x-id3", GST_RANK_PRIMARY + 2,
       id3_type_find, id3_exts, ID3_CAPS, NULL, NULL);
-  TYPE_FIND_REGISTER (plugin, "application/x-apetag", GST_RANK_PRIMARY,
+  TYPE_FIND_REGISTER (plugin, "application/x-apetag", GST_RANK_PRIMARY + 1,
       apetag_type_find, apetag_exts, APETAG_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "audio/x-ttafile", GST_RANK_PRIMARY,
       tta_type_find, tta_exts, TTA_CAPS, NULL, NULL);
@@ -2142,6 +2294,8 @@ plugin_init (GstPlugin * plugin)
   TYPE_FIND_REGISTER (plugin, "video/mpeg-stream", GST_RANK_MARGINAL,
       mpeg_video_stream_type_find, mpeg_video_exts, MPEG_VIDEO_CAPS, NULL,
       NULL);
+  TYPE_FIND_REGISTER (plugin, "video/mpeg4", GST_RANK_PRIMARY,
+      mpeg4_video_type_find, m4v_exts, MPEG_VIDEO_CAPS, NULL, NULL);
 
   /* ISO formats */
   TYPE_FIND_REGISTER (plugin, "audio/x-m4a", GST_RANK_PRIMARY, m4a_type_find,
@@ -2155,6 +2309,8 @@ plugin_init (GstPlugin * plugin)
       GST_RANK_SECONDARY, rm_exts, ".RMF", 4, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "application/x-shockwave-flash",
       GST_RANK_SECONDARY, swf_type_find, swf_exts, SWF_CAPS, NULL, NULL);
+  TYPE_FIND_REGISTER_START_WITH (plugin, "video/x-flv", GST_RANK_SECONDARY,
+      flv_exts, "FLV", 3, GST_TYPE_FIND_MAXIMUM);
   TYPE_FIND_REGISTER (plugin, "text/plain", GST_RANK_MARGINAL, utf8_type_find,
       utf8_exts, UTF8_CAPS, NULL, NULL);
   TYPE_FIND_REGISTER (plugin, "text/uri-list", GST_RANK_MARGINAL, uri_type_find,
