@@ -25,9 +25,6 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstbasetransform.h>
-#include <gst/interfaces/streamvolume.h>
-#include <gst/audio/audio.h>
-#include <gst/audio/gstaudiofilter.h>
 
 G_BEGIN_DECLS
 
@@ -36,42 +33,39 @@ G_BEGIN_DECLS
 #define GST_VOLUME(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VOLUME,GstVolume))
 #define GST_VOLUME_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VOLUME,GstVolumeClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VOLUME,GstVolume))
 #define GST_IS_VOLUME(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VOLUME))
-#define GST_IS_VOLUME_CLASS(klass) \
+#define GST_IS_VOLUME_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VOLUME))
 
 typedef struct _GstVolume GstVolume;
 typedef struct _GstVolumeClass GstVolumeClass;
 
+typedef enum {
+  GST_VOLUME_FORMAT_INT=1,
+  GST_VOLUME_FORMAT_FLOAT
+} GstVolumeFormat;
+
 /**
  * GstVolume:
  *
  * Opaque data structure.
  */
 struct _GstVolume {
-  GstAudioFilter element;
+  GstBaseTransform element;
 
-  void (*process)(GstVolume*, gpointer, guint);
+  void (*process)(GstVolume*, GstClockTime, gpointer, gint);
 
   gboolean mute;
-  gfloat volume;
+  gint   volume_i, real_vol_i; /* the _i(nt) values get synchronized with the */
+  gfloat volume_f, real_vol_f; /* _f(loat) values on each update */
 
-  gboolean current_mute;
-  gfloat current_volume;
-
-  gint   current_vol_i32;
-  gint   current_vol_i24; /* the _i(nt) values get synchronized with the */
-  gint   current_vol_i16; /* the _i(nt) values get synchronized with the */
-  gint   current_vol_i8;   /* the _i(nt) values get synchronized with the */
-  
   GList *tracklist;
-  gboolean negotiated;
 };
 
 struct _GstVolumeClass {
-  GstAudioFilterClass parent_class;
+  GstBaseTransformClass parent_class;
 };
 
 GType gst_volume_get_type (void);
