@@ -237,8 +237,6 @@ gst_xvimage_buffer_destroy (GstXvImageBuffer * xvimage)
       XSync (xvimagesink->xcontext->disp, FALSE);
       shmdt (xvimage->SHMInfo.shmaddr);
     }
-    if (xvimage->SHMInfo.shmid > 0)
-      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
     if (xvimage->xvimage)
       XFree (xvimage->xvimage);
   } else
@@ -382,9 +380,17 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   gint size;
   int (*handler) (Display *, XErrorEvent *);
   gboolean result = FALSE;
+  gboolean did_attach = FALSE;
 
   g_return_val_if_fail (xcontext != NULL, FALSE);
 
+  /* Sync to ensure any older errors are already processed */
+  XSync (xcontext->disp, FALSE);
+
+  /* Set defaults so we don't free these later unnecessarily */
+  SHMInfo.shmaddr = ((void *) -1);
+  SHMInfo.shmid = -1;
+
   /* Setting an error handler to catch failure */
   error_caught = FALSE;
   handler = XSetErrorHandler (gst_xvimagesink_handle_xerror);
@@ -393,7 +399,10 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   GST_DEBUG ("XvShmCreateImage of 1x1");
   xvimage = XvShmCreateImage (xcontext->disp, xcontext->xv_port_id,
       xcontext->im_format, NULL, 1, 1, &SHMInfo);
-  if (!xvimage) {
+
+  /* Might cause an error, sync to ensure it is noticed */
+  XSync (xcontext->disp, FALSE);
+  if (!xvimage || error_caught) {
     GST_WARNING ("could not XvShmCreateImage a 1x1 image");
     goto beach;
   }
@@ -408,9 +417,16 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   SHMInfo.shmaddr = shmat (SHMInfo.shmid, 0, 0);
   if (SHMInfo.shmaddr == ((void *) -1)) {
     GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
+    /* Clean up the shared memory segment */
+    shmctl (SHMInfo.shmid, IPC_RMID, 0);
     goto beach;
   }
 
+  /* Delete the shared memory segment as soon as we manage to attach.
+   * This way, it will be deleted as soon as we detach later, and not
+   * leaked if we crash. */
+  shmctl (SHMInfo.shmid, IPC_RMID, 0);
+
   xvimage->data = SHMInfo.shmaddr;
   SHMInfo.readOnly = FALSE;
 
@@ -419,21 +435,28 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
     goto beach;
   }
 
-  /* store whether we succeeded in result and reset error_caught */
-  result = !error_caught;
-  error_caught = FALSE;
+  /* Sync to ensure we see any errors we caused */
+  XSync (xcontext->disp, FALSE);
+
+  if (!error_caught) {
+    did_attach = TRUE;
+    /* store whether we succeeded in result */
+    result = TRUE;
+  }
 
 beach:
+  /* Sync to ensure we swallow any errors we caused and reset error_caught */
+  XSync (xcontext->disp, FALSE);
+
+  error_caught = FALSE;
   XSetErrorHandler (handler);
 
-  XSync (xcontext->disp, FALSE);
-  if (SHMInfo.shmaddr != ((void *) -1)) {
+  if (did_attach) {
     XShmDetach (xcontext->disp, &SHMInfo);
     XSync (xcontext->disp, FALSE);
-    shmdt (SHMInfo.shmaddr);
   }
-  if (SHMInfo.shmid > 0)
-    shmctl (SHMInfo.shmid, IPC_RMID, 0);
+  if (SHMInfo.shmaddr != ((void *) -1))
+    shmdt (SHMInfo.shmaddr);
   if (xvimage)
     XFree (xvimage);
   return result;
@@ -450,7 +473,6 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
 
-
   xvimage = (GstXvImageBuffer *) gst_mini_object_new (GST_TYPE_XVIMAGE_BUFFER);
 
   structure = gst_caps_get_structure (caps, 0);
@@ -501,9 +523,16 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink, GstCaps * caps)
     if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
       GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
           ("Failed to shmat: %s", g_strerror (errno)));
+      /* Clean up the shared memory segment */
+      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
       goto beach;
     }
 
+    /* Delete the shared memory segment as soon as we manage to attach.
+     * This way, it will be deleted as soon as we detach later, and not
+     * leaked if we crash. */
+    shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+
     xvimage->xvimage->data = xvimage->SHMInfo.shmaddr;
     xvimage->SHMInfo.readOnly = FALSE;
 
@@ -1804,7 +1833,8 @@ gst_xvimagesink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
       xvimagesink->xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
           GST_BUFFER_CAPS (buf));
 
-      if (!xvimagesink->xvimage)
+      if ((!xvimagesink->xvimage) ||
+          (xvimagesink->xvimage->size < GST_BUFFER_SIZE (buf)))
         goto no_image;
     }
 
@@ -1946,6 +1976,11 @@ gst_xvimagesink_buffer_alloc (GstBaseSink * bsink, guint64 offset, guint size,
     /* We found no suitable image in the pool. Creating... */
     GST_DEBUG_OBJECT (xvimagesink, "no usable image in pool, creating xvimage");
     xvimage = gst_xvimagesink_xvimage_new (xvimagesink, intersection);
+    if (xvimage->size < size) {
+      /* This image is unusable. Destroying... */
+      gst_xvimage_buffer_free (xvimage);
+      xvimage = NULL;
+    }
   }
 
   if (xvimage) {
